jQuery.validator.methods.notempty = function (value, element) {
  return value.trim().length !== 0;
};

jQuery.validator.addMethod('completeCredentials', function (value, element, params) {
  var username = $(element).parents('form').find('input[name="' + params[0] + '"]'),
    password = $(element).parents('form').find('input[name="' + params[1] + '"]');

  if ($.validator.methods.notempty(username.val(), username)) {
    return $.validator.methods.notempty(password.val(), password);
  } else if ($.validator.methods.notempty(password.val(), password)) {
    return $.validator.methods.notempty(username.val(), username);
  }
  return true
}, 'You must supply both a username and password.');

$.validator.addMethod('moreThanZero', function (value) {
  return value > 0;
}, 'The size must be an integer > 0.');

$.validator.addMethod('passwordStrength', function (value, element) {
  return this.optional(element) ||
    value.length >= 8
    && /\d/.test(value) // contains at least one digit
    && !/^[a-z0-9]+$/i.test(value); // contains at least one special character (not digit or letter);
}, 'Password should be at least 8 characters long, contain at least 1 digit and at least 1 special character.');

var validation = {
  'registration-form': {
    rules: {
      controllerAddress: {
        required: true,
        notempty: true
      },
      address: {
        required: true,
        notempty: true
      },
      key: {
        required: true,
        notempty: true
      }
    }
  },
  purge: {},
  targetCacheSize: {
    rules: {
      size: {
        required: true,
        min: 1
      }
    }
  },
  freeSpaceBufferSize: {
    rules: {
      freeSpaceBufferSize: {
        required: true,
        min: 256
      }
    }
  },
  maxArtifactSize: {
    rules: {
      maxArtifactSize: {
        required: true,
        min: 1
      }
    }
  },
  maxEntryAgeInHours: {
    rules: {
      maxEntryAgeInHours: {
        required: function (element) {
          return $("#max-entry-age-enabled").is(':checked')
        },
        min: 1
      }
    }
  },
  credentials: {
    rules: {
      primaryUsername: {
        completeCredentials: ['primaryUsername', 'primaryPassword']
      },
      primaryPassword: {
        completeCredentials: ['primaryUsername', 'primaryPassword']
      },
      alternateUsername: {
        completeCredentials: ['alternateUsername', 'alternatePassword']
      },
      alternatePassword: {
        completeCredentials: ['alternateUsername', 'alternatePassword']
      }
    },
    groups: {
      primary: 'primaryUsername primaryPassword',
      alternate: 'alternateUsername alternatePassword'
    }
  },
  'ui-access-type':{
    rules: {
      'username': {
        required: true,
        minlength: 3
      },
      'password': {
        required: true,
        minlength: 8,
        passwordStrength: true
      },
      'confirmPassword': {
        equalTo : "#ui-secured-password-input"
      }
    }
  }
};

var app = function () {
  var bindValidation = function () {
    $('form').each(function () {
      var form = $(this);
      $(this).validate({
        ignore: '',
        onfocusout: false,
        onkeyup: false,
        errorPlacement: function (error, element) {
        },
        showErrors: function (errorMap, errorList) {
          var arrayLength = errorList.length;
          for (var i = 0; i < arrayLength; i++) {
            if (i === 0) {
              errorMsg(errorList[i].message, form.find('.msg'));
            }
            $(errorList[i].element).addClass('error');
          }
          this.defaultShowErrors();
        },
        highlight: function (element, errorClass) {
          $(element).addClass(errorClass);
        },
        unhighlight: function (element, errorClass) {
          $(element).removeClass(errorClass);
        },
        rules: validation[$(this).attr('id')].rules,
        groups: validation[$(this).attr('id')].groups,
        submitHandler: function (form, event) {
          event.preventDefault();
          var $form = $(form)
          jQuery.post($form.attr('action'), $form.serialize())
            .done(function () {
              if ($form.attr('id') === 'registration-form') {
                $('input[name=secret]').attr('placeholder', '(hidden)').val('');
              } else if ($form.attr('id') === 'ui-access-type') {
                InsecureWarningBanner.updateUiAccess($form.find('input[name="uiAccessType"]:checked').val() === 'OPEN');
              }
              successMsg($form.data('successMsg'), $form.find('.msg'));
              var callback = $form.data('onSubmit');
              if (callback) {
                callback();
              }
            })
            .fail(function () {
              errorMsg($form.data('failureMsg'), $form.find('.msg'));
            });
        }
      });
    });
  };

  jQuery(function () {
    bindAccordion();
    updateUi();
    fixFocus();
    bindValidation();
    bindUiUsernamePassword();
  });

  var updateConnectionStatus = function () {
    return jQuery.get(appPath() + '/status')
      .done(function (data) {
        var status = jQuery('#status');
        var html = status.html();
        if (html !== data.message) {
          status.removeClass().addClass(data.error ? 'error' : 'success');
          status.html(data.message);
        }
      })
      .fail(function (e) {
        console.log('failed to update status', e);
      });
  };

  var updateUi = function () {
    jQuery.get(appPath() + '/snapshot')
      .done(function (data) {
        Object.keys(data).forEach(function (key) {
          var value = data[key];
          var element = $('.refreshable.' + key);
          var updated = false;
          if (element.text() !== '' + value) {
            element.text(value);
            updated = true;
          }
          if (key === 'empty') {
            if (value) {
              $('form#purge input').attr('disabled', 'yes');
            }
            else {
              $('form#purge input').removeAttr('disabled');
            }
          }
          if (key === 'storageHealth') {
            var storageHealth = $('.refreshable.storageHealthMessage')
            if (!storageHealth.attr("class").includes(value)) {
              storageHealth
                .removeClass("error")
                .removeClass("warning")
                .removeClass("ok")
                .addClass(value);
            }
          }
          if (updated) {
            element.fadeTo(50, 0.3, function () {
              $(this).fadeTo(200, 1.0);
            });
          }
        });
      })
      .fail(function (e) {
        console.log('failed to refresh from snapshot', e);
      });

    updateConnectionStatus()
      .always(function () {
        setTimeout(updateUi, 2000)
      });
  };

  $("form#registration-form").data("onSubmit", function() {
    $("#status").html("<span class='connecting'>Connecting…");
  });

  var fixFocus = function () {
    // If the user is tabbing around, we want to show the focus ring
    document.addEventListener('keydown', function (e) {
      if (e.keyCode === 9) {
        $('body').addClass('show-focus-outlines');
      }
    });
    // But if they're clicking about, we don't
    document.addEventListener('click', function (e) {
      $('body').removeClass('show-focus-outlines');
    });
  };

  var bindAccordion = function () {
    var clickHandler = function (e) {
      if (e.type === 'keydown' && e.key !== ' ') {
        return;
      }
      // collapse all first
      $('section.accordion').each(function () {
        var acc = $(this);
        if (!acc.hasClass('collapsed')) {
          acc.find('.content').slideUp(function () {
            acc.addClass('collapsed');
          });
        }
      });

      var content = $(this).parent().find('.content');
      content.parent().removeClass('expansion-done');
      if (content.parent().hasClass('collapsed')) {
        content.slideDown(300, function () {
          content.parent().addClass('expansion-done');
        });
        content.parent().removeClass('collapsed');
      } else {
        content.slideUp(300, function () {
          content.parent().addClass('expansion-done');
        });
        content.parent().addClass('collapsed');
      }
    };

    $('section.accordion > .accordion-toggle:first-child').on('click keydown', clickHandler);
  };

  var bindUiUsernamePassword = function () {

    var showOrHideUsernamePasswordFields = function () {
      if ($('#ui-access-type-secured-radio-btn').is(':checked')) {
        $('#ui-secured-username-input').prop('disabled', false);
        $('#ui-secured-password-input').prop('disabled', false);
        $('#ui-secured-password-confirm-input').prop('disabled', false);
        $('#user-pass-fields').slideDown(300);
      } else {
        $('#user-pass-fields').slideUp(300);
        $('#ui-secured-username-input').val('');
        $('#ui-secured-password-input').val('');
        $('#ui-secured-password-confirm-input').val('');
        $('#ui-secured-username-input').prop('disabled', true);
        $('#ui-secured-password-input').prop('disabled', true);
        $('#ui-secured-password-confirm-input').prop('disabled', true);
      }
    }

    $('input[type="radio"][name="uiAccessType"]')
      .on('change', showOrHideUsernamePasswordFields);

  }
};

function ajax(url, method, payload) {
  return jQuery.ajax({
    method: method,
    url: url,
    data: payload,
    beforeSend: function(request) {
      request.setRequestHeader("X-Gradle-Enterprise-Ajax-Request", "1");
    }
  });
}

const InsecureWarningBanner = (() => {
  let self = {
    showAnonUiAccessWarning: undefined,
    showAnonCacheWriteAccessWarning: undefined
  };

  const update = () => $('#InsecureWarningBanner')
      .toggleClass('hidden', !self.showAnonUiAccessWarning && !self.showAnonCacheWriteAccessWarning)
      .find('#InsecureWarningBannerMessage')
      .powerTip({
        placement: 's',
        mouseOnToPopup: true,
        intentPollInterval: 1,
        intentSensitivity: 1000,
        fadeInTime: 300,
        fadeOutTime: 300,
        popupClass: "InsecureWarningBanner__tooltip WarningBanner__tooltip"
      })
      .data('powertip', function() {
        const messages = [];
        if (self.showAnonUiAccessWarning) {
          messages.push('<p id="anonUiAccessWarning">Access to this page does not require credentials, which may allow anyone to change the cache node configuration.</p>');
        }
        if(self.showAnonCacheWriteAccessWarning) {
          messages.push('<p id="anonCacheWriteAccessWarning">Writing to the build cache does not require credentials, which may allow anyone to write malicious cache entries.</p>');
        }
        if (self.showAnonUiAccessWarning || self.showAnonCacheWriteAccessWarning) {
          messages.push('<p>Please consult the <a href="https://docs.gradle.com/build-cache-node/#insecure_configuration">User Manual</a> for more information.</p>');
        }
        return messages.join('');
      });

  return {
    initialize: (showAnonUiAccessWarning, showAnonCacheWriteAccessWarning) => {
      self.showAnonUiAccessWarning = showAnonUiAccessWarning;
      self.showAnonCacheWriteAccessWarning = showAnonCacheWriteAccessWarning;
      update();
    },

    updateAnonymousCacheAccess: (isReadWrite) => {
      self.showAnonCacheWriteAccessWarning = isReadWrite;
      update();
    },

    updateUiAccess: (isOpen) => {
      self.showAnonUiAccessWarning = isOpen;
      update()
    }
  }
})();

const InaccessibleWarningBanner = (() => {
  let self = {
    showEffectivelyInaccessibleWarning: undefined
  };

  const update = () => $('#InaccessibleWarningBanner')
    .toggleClass('hidden', !self.showEffectivelyInaccessibleWarning)
    .find('#InaccessibleWarningBannerMessage')
    .powerTip({
      placement: 's',
      mouseOnToPopup: true,
      intentPollInterval: 1,
      intentSensitivity: 1000,
      fadeInTime: 300,
      fadeOutTime: 300,
      popupClass: "InaccessibleWarningBanner__tooltip WarningBanner__tooltip"
    })
    .data('powertip', function() {
      const messages = [];
      messages.push('<p id="cacheEffectivelyInaccessibleWarning">The build cache is inaccessible because there are no configured users and the anonymous access is disabled.</p>');
      messages.push('<p id="cacheEffectivelyInaccessibleWarning2">Use “Build cache” » “Settings” » “Cache access control” to add users and/or change anonymous access settings.</p>');
      return messages.join('');
    });

  return {
    initialize: (isEffectivelyInaccessible) => {
      self.showEffectivelyInaccessibleWarning = isEffectivelyInaccessible;
      update();
    },

    updateEffectivelyInaccessibleCacheAccess: (isEffectivelyInaccessible) => {
      self.showEffectivelyInaccessibleWarning = isEffectivelyInaccessible;
      update();
    },

  }
})();

function msg(cls, html, anchor) {
  $("#msg").remove();
  var msgBox = $("<div id='msg'/>").insertBefore(anchor || "#main");
  msgBox.removeClass().addClass(cls).hide().fadeIn(300).html(html);
}
function successMsg(html, anchor) {
  msg("success", html, anchor);
}
function errorMsg(html, anchor) {
  msg("error", html, anchor);
}

function appPath() {
  var appPath = window.location.pathname
  if (!appPath.startsWith("/")) {
    appPath = "/" + appPath;
  }
  if (appPath.endsWith("/")) {
    appPath = appPath.substring(0, appPath.length - 1);
  }
  return appPath
}
